/*
 * Copyright (C) 2004 Domenico Delle Side
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include "getphoto.h"

#include <unistd.h>
#include <sys/mount.h>
#include <time.h>


DIR *Opendir(const char *dirname)
{
  DIR *dir;

  if ( NULL == (dir = opendir(dirname)) )
    print_error(GT_("Can't open %s: %m\n"), dirname);

  return dir;
}


int Chdir(const char *path)
{
  if ( 0 > chdir(path) )
    {
      print_error(GT_("Can't access to %s: %m\n"), path);
      return -1;
    }

  return 1;
}


int my_Fchdir(DIR *dir)
{
  if ( 0 > fchdir(dirfd(dir)) )
    {   
      print_error(GT_("Can't come back: %m\n"), dirfd(dir));
      return -1;
    }

  return 1;
}


static int my_Mount(const char *device, const char *dir, const char *fstype)
{
  if ( 0 > mount(device, dir, fstype, MS_MGC_VAL, "") )
    {
      print_error(GT_("Can't mount %s: %m\n"), device);
      return -1;
    }

  return 1;
}


static char *get_date(void)
{
  time_t now = time(NULL);
  /* 
   * This could lead to security problems, we must try to fix it in
   * the near future.
   *
   */
  char *date = Calloc((size_t)9, sizeof(char));

  
  if ( 0 == strftime(date, (size_t)9, "%d%b%y", localtime(&now)) )
    return NULL;

  return date;
}


static char *update_dest_dir(char *old)
{
  char *tmp, *new_dir;
  size_t length;
      
  new_dir = get_date();
  length = strlen(old) + strlen(new_dir) + (size_t)2;
  tmp = Calloc(length, sizeof(char));
  snprintf(tmp, length, "%s/%s", old, new_dir);
  free(new_dir);

  return tmp;
}


int scan_camera(struct config *conf)
     /*
      * This function works almost properly but it's too stupid.
      * FIXME: change it in the near future.
      *
      */
{
  DIR *photo_dir = NULL;
  DIR *saved_dir = NULL;
  struct dirent *photo_item;
  char *total_photo_path, *final_dest_dir;
  unsigned int number_of_items = 0;
  size_t len = strlen(conf->mount_point) + strlen(conf->photo_path) + (size_t)2;
  int mounted = 0;


  if ( NULL == (saved_dir = Opendir(".")) )
    goto end;
    
  if ( 0 > my_Mount(conf->device, conf->mount_point, conf->fs_type) )
    goto end;
  else mounted = 1;

  total_photo_path = Calloc(len, sizeof(char));
  snprintf(total_photo_path, len, "%s/%s", conf->mount_point, 
	   conf->photo_path);

  if ( !Chdir(total_photo_path) )
    goto end;
  
  if ( NULL == (photo_dir = Opendir(total_photo_path)) )
    goto end;

  if ( 1 == conf->usedate )
     {
       final_dest_dir = update_dest_dir(conf->dest_dir);
       free(conf->dest_dir);
       conf->dest_dir = mystrndup(final_dest_dir, strlen(final_dest_dir));
       free(final_dest_dir);
     }


  while ( NULL != (photo_item = readdir(photo_dir)) )
    {
      if ( '.' == *photo_item->d_name )
	continue;

      if ( 0 > copy_file(photo_item->d_name, conf->dest_dir) )
	{
	  print_error(GT_("Errors copying %s to %s\n"), 
		      photo_item->d_name, conf->dest_dir);
	  continue;
	}
      else
	{
	  if ( 0 == conf->preserve )
	    unlink(photo_item->d_name);

	  number_of_items++;
	}
    }

  free(total_photo_path);
  closedir(photo_dir);
  
 end:
  my_Fchdir(saved_dir);
  closedir(saved_dir);

  if ( mounted )
    {
#ifdef HAVE_UMOUNT2
      if ( 0 > umount2(conf->mount_point, MNT_FORCE) )
#else
      if ( 0 > umount(conf->mount_point) )
#endif
	print_error("Can't unmount: %m\n");
    }
  else
    return -1;
  
  return number_of_items;
}
