/*
 * Copyright (C) 2004 Domenico Delle Side
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include "config.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <stdarg.h>
#include <ctype.h>
#include <sys/types.h>
#include <dirent.h>

#ifdef ENABLE_NLS
#include <libintl.h>
#define GT_(String) gettext((String))
#else
#define GT_(String) (String)
#endif


#define CONF_FILE ".getphoto.rc"
#define RESIZED_PATH "resized"
#define MAXLEN 1024

#ifdef HAVE_JPEGLIB_H
struct image {
  unsigned int img_quality;
  unsigned int img_w;
  unsigned int img_h;
};
#endif


struct config {
  char *device;
  char *fs_type;
  char *mount_point;
  char *photo_path;
  char *dest_dir;
  int preserve;
  int usedate;
  int resize;
#ifdef HAVE_JPEGLIB_H
  struct image img;
#endif
};


/* Errors handling functions */
void print_error( char *, ... );
void print_errno( void );
int print_exit( char *, ... );
int exit_errno( void );


/* Memory functions */
char *mystrndup(char *, size_t);
void *Calloc(size_t , size_t);


/* Parsing functions */
struct config *parse_config(char *);
void free_options(struct config *);


/* Copy functions */
int copy_file(char *, char *);


/* Directory handling functions */
int scan_camera(struct config *);
void Mkdir(const char *);
int check_dir(const char *);
DIR * Opendir(const char *);
int my_Fchdir(DIR *);
int Chdir(const char *);


/* Ownership changing functions */
int change_user(const char *);


#ifdef HAVE_JPEGLIB_H
/* Image handling functions */
int make_resized_images(struct config *);
#endif
